@@ -1,4 +1,4 @@
-# $Id: progressbar.rb 438 2008-09-02 01:49:07Z helixblue $
+# $Id: progressbar.rb 554 2010-01-27 07:06:17Z helixblue $
 
 class ProgressBar
   def initialize(start, max, name)
@@ -34,23 +34,28 @@ class ProgressBar
     end
         
     # if the value is 0 or less, don't bother to print up a bar.
-    if (@value < 1)
+    if not @value or @value < 1
       return
     end
+
+    if @value == @max
+      meter = '==='
+    else
+      percentage = (@value.to_f / @max.to_f) * 100
+      bardiv = percentage.divmod(33.333333333333333333333333333333333333)
         
-    percentage = (@value.to_f / @max.to_f) * 100
-    bardiv = percentage.divmod(33.333333333333333333333333333333333333)
-        
-    fullbars = bardiv[0]
-    minibars = bardiv[1] / (16.66666666666666666666666666667).round
-    # "p=#{percentage} f=#{fullbars} m=#{minibars}"
-    meter = "=" * fullbars + "-" * minibars
-    meter = meter.ljust(3)
-        
+      fullbars = bardiv[0]
+      minibars = bardiv[1] / (16.66666666666666666666666666667).round
+      # "p=#{percentage} f=#{fullbars} m=#{minibars}"
+      meter = "=" * fullbars + "-" * minibars
+      meter = meter.ljust(3)
+    end
+
     if (@valueText)
       puts "[#{meter}] (#{@value}/#{@max}) #{@name}: #{@valueText}"
     else
       puts "[#{meter}] (#{@value}/#{@max}) #{@name}"
     end
+    STDOUT.flush
   end
 end
