@@ -1,8 +1,8 @@
-# $Id: searchcode.rb 426 2008-09-01 12:24:15Z helixblue $
+# $Id: searchcode.rb 527 2010-01-24 19:26:12Z helixblue $
 
 class SearchCode
   include Common
-  include Display
+  include Messages
     
   # output from tools/countryrip.rb
   $idHash = Hash.new
