@@ -1,7 +1,7 @@
 # This is our dump templating engine. It only handles really simple text formats at the moment.
 # GeoToad 4.0 will have a plugins architecture that replaces this.
 #
-# $Id: templates.rb 484 2009-06-30 13:19:48Z helixblue $
+# $Id: templates.rb 545 2010-01-25 12:39:28Z helixblue $
 
 $Format = {
   'gpspoint'    => {
@@ -33,81 +33,43 @@ $Format = {
     'ext'        => 'gpx',
     'mime'    => 'text/ascii',
     'desc'    => 'GPX Geocaching XML',
-    'spacer'    => "\n",
-    'templatePre' => "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
-      "<gpx xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" version=\"1.0\" creator=\"GeoToad\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd http://www.groundspeak.com/cache/1/0 http://www.groundspeak.com/cache/1/0/cache.xsd\" xmlns=\"http://www.topografix.com/GPX/1/0\">\n" +
-      "<desc><%outEntity.title%></desc>\n" +
-      "<author>GeoToad #{$VERSION}</author>\n" +
-      "<time>" + Time.new.gmtime.strftime("%Y-%m-%dT%H:%M:%S")  + ".0000000-00:00</time>\n" +
-      "<keywords>cache, geocache, groundspeak, geotoad</keywords>\n",
+    'spacer'    => "\r\n",
+    'templatePre' => "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" +
+      "<gpx xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" version=\"1.0\" creator=\"GeoToad\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd http://www.groundspeak.com/cache/1/0 http://www.groundspeak.com/cache/1/0/cache.xsd\" xmlns=\"http://www.topografix.com/GPX/1/0\">\r\n" +
+      "<desc><%outEntity.title%></desc>\r\n" +
+      "<author>GeoToad #{$VERSION}</author>\r\n" +
+      "<email>geotoad@toadstool.se</email>\r\n" +
+      "<time>" + Time.new.gmtime.strftime("%Y-%m-%dT%H:%M:%S")  + ".0000000-00:00</time>\r\n" +
+      "<keywords>cache, geocache, groundspeak, geotoad</keywords>\r\n",
 
-    'templateWP'    => "<wpt lat=\"<%wp.latdata%>\" lon=\"<%wp.londata%>\">\n" +
-      "  <time><%out.XMLDate%></time>\n" +
-      "  <name><%outEntity.id%></name>\n" +
-      "  <desc><%wpEntity.name%> by <%wpEntity.creator%>, <%wp.type%> (<%wp.difficulty%>/<%wp.terrain%>)</desc>\n" +
-      "  <url><%out.url%></url>\n" +
-      "  <urlname><%wpEntity.name%> by <%wpEntity.creator%></urlname>\n" +
-      "  <sym>Geocache</sym><type>Geocache</type>\n" +
-      "  <groundspeak:cache id=\"<%wp.sid%>\" available=\"True\" archived=\"False\" xmlns:groundspeak=\"http://www.groundspeak.com/cache/1/0\">\n" +
-      "  <groundspeak:date><%out.XMLDate%></groundspeak:date>\n" +
-      "  <groundspeak:name><%wpEntity.name%></groundspeak:name>\n" +
-      " <groundspeak:full_name><%wpEntity.name%></groundspeak:full_name>\n"+
-      "  <groundspeak:placed_by><%wpEntity.creator%></groundspeak:placed_by>\n" +
-      "  <groundspeak:owner id=\"999\"><%wpEntity.creator%></groundspeak:owner>\n" +
-      "  <groundspeak:type><%wp.type%></groundspeak:type>\n" +
-      "  <groundspeak:container><%wp.size%></groundspeak:container>\n" +
-      "  <groundspeak:difficulty><%wp.difficulty%></groundspeak:difficulty>\n" +
-      "  <groundspeak:terrain><%wp.terrain%></groundspeak:terrain>\n" +
-      "  <groundspeak:country><%wpEntity.country%></groundspeak:country>\n" +
-      "  <groundspeak:state><%wpEntity.state%></groundspeak:state>\n" +
-      "  <groundspeak:short_description html=\"False\"><%wpEntity.shortdesc%></groundspeak:short_description>\n" +
-      "  <groundspeak:long_description html=\"True\"><%wpEntity.longdesc%></groundspeak:long_description>\n" +
-      "  <groundspeak:encoded_hints><%outEntity.hintdecrypt%></groundspeak:encoded_hints>\n" +
-      "  <groundspeak:logs>\n" +
-      "  <%out.gpxlogs%>\n" +
-      "  </groundspeak:logs>\n" +
-      "  <groundspeak:travelbugs />\n" +
-      "  </groundspeak:cache>\n" +
-      "</wpt>\n",
-    'templatePost'    => " </gpx>\n"
-  },
-
-  # Modified GPX XML for Pathaway. Here, the <desc> tag will include the longdescription!
-  'gpx_pa'    => {
-    'ext'        => 'gpx',
-    'mime'    => 'text/ascii',
-    'desc'    => 'GPX XML, modified for Pathaway',
-    'spacer'    => "\n",
-    'templatePre' => "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
-      "<gpx xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" version=\"1.0\" creator=\"GeoToad\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd http://www.groundspeak.com/cache/1/0 http://www.groundspeak.com/cache/1/0/cache.xsd\" xmlns=\"http://www.topografix.com/GPX/1/0\">\n" +
-      "<desc><%outEntity.title%></desc>\n" +
-      "<author>GeoToad #{$VERSION}</author>\n" +
-      "<time>" + Time.new.gmtime.strftime("%Y-%m-%dT%H:%M:%S")  + ".0000000-00:00</time>\n",
-
-    'templateWP'    => "<wpt lat=\"<%wp.latdata%>\" lon=\"<%wp.londata%>\">\n" +
-      "  <time><%out.XMLDate%></time>\n" +
-      "  <name><%outEntity.id%></name>\n" +
-      "  <desc><%outEntity.wid%>: <%wpEntity.name%>\n=== <%wp.type%> (<%wp.difficulty%>/<%wp.terrain%>/C:<%wp.size%>) by <%wpEntity.creator%>\n\n<%wpEntity.longdesc%>\n<%outEntity.hint%></desc>\n" +
-      "  <url><%out.url%></url>\n" +
-      "  <urlname><%wpEntity.name%> by <%wpEntity.creator%></urlname>\n" +
-      "  <sym>Geocache</sym><type>Geocache</type>\n" +
-      "  <groundspeak:cache id=\"<%wp.sid%>\" available=\"True\" archived=\"False\" xmlns:groundspeak=\"http://www.groundspeak.com/cache/1/0\">\n" +
-      "  <groundspeak:date><%out.XMLDate%></groundspeak:date>\n" +
-      "  <groundspeak:name><%wpEntity.name%></groundspeak:name>\n" +
-      " <groundspeak:full_name><%wpEntity.name%></groundspeak:full_name>\n"+
-      "  <groundspeak:placed_by><%wpEntity.creator%></groundspeak:placed_by>\n" +
-      "  <groundspeak:owner id=\"999\"><%wpEntity.creator%></groundspeak:owner>\n" +
-      "  <groundspeak:type><%wp.type%></groundspeak:type>\n" +
-      "  <groundspeak:container><%wp.size%></groundspeak:container>\n" +
-      "  <groundspeak:difficulty><%wp.difficulty%></groundspeak:difficulty>\n" +
-      "  <groundspeak:terrain><%wp.terrain%></groundspeak:terrain>\n" +
-      "  <groundspeak:country><%wpEntity.country%></groundspeak:country>\n" +
-      "  <groundspeak:state><%wpEntity.state%></groundspeak:state>\n" +
-      "  <groundspeak:encoded_hints><%outEntity.hintdecrypt%></groundspeak:encoded_hints>\n" +
-      "  <groundspeak:travelbugs />\n" +
-      "  </groundspeak:cache>\n" +
-      "</wpt>\n",
-    'templatePost'    => " </gpx>\n"
+    'templateWP'    => "<wpt lat=\"<%wp.latdata%>\" lon=\"<%wp.londata%>\">\r\n" +
+      "  <time><%out.XMLDate%></time>\r\n" +
+      "  <name><%outEntity.id%></name>\r\n" +
+      "  <desc><%wpEntity.name%> by <%wpEntity.creator%>, <%wp.type%> (<%wp.difficulty%>/<%wp.terrain%>)</desc>\r\n" +
+      "  <url><%out.url%></url>\r\n" +
+      "  <urlname><%wpEntity.name%> by <%wpEntity.creator%>, <%wp.fulltype%> (<%wp.difficulty%>/<%wp.terrain%>)</urlname>\r\n" +
+      "  <sym>Geocache</sym><type>Geocache</type>\r\n" +
+      "  <groundspeak:cache id=\"<%wp.sid%>\" available=\"True\" archived=\"False\" xmlns:groundspeak=\"http://www.groundspeak.com/cache/1/0\">\r\n" +
+      "  <groundspeak:name><%wpEntity.name%></groundspeak:name>\r\n" +
+      " <groundspeak:full_name><%wpEntity.name%></groundspeak:full_name>\r\n"+
+      "  <groundspeak:placed_by><%wpEntity.creator%></groundspeak:placed_by>\r\n" +
+      "  <groundspeak:owner id=\"<%wpEntity.creator_id%>\"><%wpEntity.creator%></groundspeak:owner>\r\n" +
+      "  <groundspeak:type><%wp.fulltype%></groundspeak:type>\r\n" +
+      "  <groundspeak:container><%wp.size%></groundspeak:container>\r\n" +
+      "  <groundspeak:difficulty><%wp.difficulty%></groundspeak:difficulty>\r\n" +
+      "  <groundspeak:terrain><%wp.terrain%></groundspeak:terrain>\r\n" +
+      "  <groundspeak:country><%wpEntity.country%></groundspeak:country>\r\n" +
+      "  <groundspeak:state><%wpEntity.state%></groundspeak:state>\r\n" +
+      "  <groundspeak:short_description html=\"True\"><%wpEntity.additional_raw%><%wpEntity.shortdesc%></groundspeak:short_description>\r\n" +
+      "  <groundspeak:long_description html=\"True\"><%wpEntity.longdesc%></groundspeak:long_description>\r\n" +
+      "  <groundspeak:encoded_hints><%outEntity.hintdecrypt%></groundspeak:encoded_hints>\r\n" +
+      "  <groundspeak:logs>\r\n" +
+      "  <%out.gpxlogs%>\r\n" +
+      "  </groundspeak:logs>\r\n" +
+      "  <groundspeak:travelbugs />\r\n" +
+      "  </groundspeak:cache>\r\n" +
+      "</wpt>\r\n",
+    'templatePost'    => " </gpx>\r\n"
   },
 
   'html'    => {
@@ -120,11 +82,11 @@ $Format = {
       "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n" + "</head>\n" +
       "<body link=\"#000099\" vlink=\"#000044\" alink=\"#000099\">\n" +
       "<h3><%out.title%></h3>" +
-      "<b><font color=\"#11CC11\">$</font></b> have travelbugs&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#9900CC\">@</font></b> never been found&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#229999\">%</font></b> terrain rating of 3.5+&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#BB0000\">&gt;</font></b> difficulty rating of 3.5+&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#E2BF2B\">#</font></b> fun factor rating of 3.5+<br>" +
+      "<b><font color=\"#11CC11\">&euro;</font></b> have travelbugs&nbsp;&nbsp;&nbsp;" +
+      "<b><font color=\"#9900CC\">&infin;</font></b> never been found&nbsp;&nbsp;&nbsp;" +
+      "<b><font color=\"#999922\">&sect;</font></b> terrain rating of 3.5+&nbsp;&nbsp;&nbsp;" +
+      "<b><font color=\"#440000\">&uarr;</font></b> difficulty rating of 3.5+&nbsp;&nbsp;&nbsp;" +
+      "<b><font color=\"#BB2222\">&hearts;</font></b> fun factor rating of 3.5+<br>" +
       # Not yet ready for consumption
     #"<b><font color=\"#333333\">--</font></b> comments seem negative&nbsp;&nbsp;&nbsp;" +
     #"<b><font color=\"#000000\">++</font></b> comments seem very positive" +
@@ -136,76 +98,33 @@ A|B|C|D|E|F|G|H|I|J|K|L|M
 N|O|P|Q|R|S|T|U|V|W|X|Y|Z</pre></font><br>",
     'templateIndex' => "* <a href=\"#<%out.wid%>\"><%wpEntity.name%></a><br>",
     'templateWP'    =>
-      "\n\n<hr noshade size=\"1\">\n<a name=\"<%out.wid%>\"></a><font color=\"#000099\"><big><b><a href=\"<%out.url%>\"><%wpEntity.name%>&nbsp;<%out.symbols%></a></b></big></font><br>\n" +
+      "\n\n<hr noshade size=\"1\">\n<a name=\"<%out.wid%>\"></a><font color=\"#000099\"><big><b><a href=\"<%out.url%>\"><%wp.name%>&nbsp;<%out.symbols%></a></b></big></font><br>\n" +
       "<font color=\"#555555\"><b><%wpEntity.creator%></b></font>, <%wp.latwritten%> <%wp.lonwritten%> (<%out.location%>)<br>" +
       "<font color=\"#339933\"><%wp.type%> (<%wp.size%>) D<%wp.difficulty%>/T<%wp.terrain%> F:<%wp.funfactor%> <%out.relativedistance%><br>" +
-      "placed: <%wp.cdate%> last comment: <%wp.mdays%> days ago (<%wp.comment0Type%>)</font><br>" +
-      "<p><%outEntity.details%></p>\n" +
+      "placed: <%out.cdate%> last comment: <%wp.comment0DaysAgo%> days ago (<%wp.comment0Type%>)</font><br>" +
+      "<p><%wp.additional_raw%><%wp.shortdesc%></p>\n" +
+      "<p><%wp.longdesc%></p>\n" +
       "<p><font color=\"#555555\"><%outEntity.hint%></font></p>\n",
     'templatePost'    => "</body></html>"
   },
 
-
-  'html-decrypt'    => {
-    'ext'        => 'html',
-    'mime'    => 'text/html',
-    'desc'    => 'Simple HTML with hint decrypted',
-    'spacer'    => "<br>&nbsp;\n",
-    'templatePre' => "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n" +
-      "<html><head>\n<title><%out.title%></title>\n" +
-      "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n" + "</head>\n" +
-      "<body link=\"#000099\" vlink=\"#000044\" alink=\"#000099\">\n" +
-      "<h3><%out.title%></h3>" +
-      "<b><font color=\"#11CC11\">$</font></b> have travelbugs&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#9900CC\">@</font></b> never been found&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#229999\">%</font></b> terrain rating of 3.5+&nbsp;&nbsp;&nbsp;" +
-      "<b><font color=\"#BB0000\">&gt;</font></b> difficulty rating of 3.5+&nbsp;&nbsp;&nbsp;",
-    "<b><font color=\"#E2BF2B\">#</font></b> fun factor rating of 3.5+<br>" +
-      # Not yet ready for consumption
-    #"<b><font color=\"#333333\">--</font></b> comments seem negative&nbsp;&nbsp;&nbsp;" +
-    #"<b><font color=\"#000000\">++</font></b> comments seem very positive" +
-    'templateIndex' => "* <a href=\"#<%out.wid%>\"><%wpEntity.name%></a><br>",
-    'templateWP'    =>
-      "\n\n<hr noshade size=\"1\">\n<a name=\"<%out.wid%>\"></a><font color=\"#000099\"><big><b><a href=\"<%out.url%>\"><%wpEntity.name%>&nbsp;<%out.symbols%></a></b></big></font><br>\n" +
-      "<font color=\"#555555\"><b><%wpEntity.creator%></b></font>, <%wp.latwritten%> <%wp.lonwritten%> (<%out.location%>)<br>" +
-      "<font color=\"#339933\"><%wp.type%> (<%wp.size%>) D<%wp.difficulty%>/T<%wp.terrain%> F:<%wp.funfactor%> <%out.relativedistance%><br>" +
-      "placed: <%wp.cdate%> last comment: <%wp.mdays%> days ago (<%wp.comment0Type%>)</font><br>" +
-      "<p><%outEntity.details%></p>\n" +
-      "<p><font color=\"#555555\"><%out.hintdecrypt%></font></p>\n",
-    'templatePost'    => "</body></html>"
-  },
-
+  # TODO(thomas): Add html-decrypt and text-decrypt back without too much duplication.
 
   'text'    => {
     'ext'        => 'txt',
     'mime'    => 'text/plain',
     'desc'    =>     'Plain ASCII',
-    'spacer'    => "\n",
-    'templatePre' =>  "== <%out.title%>\n\nDecryption Key (letter above equals below, and vice versa)\n\nA|B|C|D|E|F|G|H|I|J|K|L|M\n-------------------------\nN|O|P|Q|R|S|T|U|V|W|X|Y|Z\n\n\n",
-    'templateWP'    => "----------------------------------------------------------------\n" +
-      "* <%wp.name%>\" (<%out.wid%>) by <%wp.creator%>\n" +
-      "Difficulty: <%wp.difficulty%>, Terrain: <%wp.terrain%>, FunFactor: <%wp.funfactor%>\n" +
-      "Lat: <%wp.latwritten%> Lon: <%wp.lonwritten%>\n" +
-      "Type: <%wp.type%> (<%wp.size%>) <%out.relativedistance%>\n" +
-      "Creation: <%wp.cdate%>, Last comment: <%wp.mdays%> days ago (<%wp.comment0Type%>)\n" +
-      "\n<%out.details%>\n" +
-      "\n<%out.hint%>\n\n\n\n"
-  },
-
-  'text-decrypt'    => {
-    'ext'        => 'txt',
-    'mime'    => 'text/plain',
-    'desc'    =>     'Plain ASCII with hint decrypted',
-    'spacer'    => "\n",
-    'templatePre' =>  "== <%out.title%>\n\nDecryption Key (letter above equals below, and vice versa)\n\nA|B|C|D|E|F|G|H|I|J|K|L|M\n-------------------------\nN|O|P|Q|R|S|T|U|V|W|X|Y|Z\n\n\n",
-    'templateWP'    => "----------------------------------------------------------------\n" +
-      "* <%wp.name%>\" (<%out.wid%>) by <%wp.creator%>\n" +
-      "Difficulty: <%wp.difficulty%>, Terrain: <%wp.terrain%>, FunFactor: <%wp.funfactor%>\n" +
-      "Lat: <%wp.latwritten%> Lon: <%wp.lonwritten%>\n" +
-      "Type: <%wp.type%> (<%wp.size%>) <%out.relativedistance%>\n" +
-      "Creation: <%wp.cdate%>, Last comment: <%wp.mdays%> days ago (<%wp.comment0Type%>)\n" +
-      "\n<%out.details%>\n" +
-      "\n<%out.hintdecrypt%>\n\n\n\n"
+    'spacer'    => "\r\n",
+    'templatePre' =>  "== <%out.title%>\r\n\r\nDecryption Key (letter above equals below, and vice versa)\r\n\r\nA|B|C|D|E|F|G|H|I|J|K|L|M\r\n-------------------------\r\nN|O|P|Q|R|S|T|U|V|W|X|Y|Z\r\n\r\n\r\n",
+    'templateWP'    => "----------------------------------------------------------------\r\n" +
+      "* <%wpText.name%> (<%out.wid%>) by <%wpText.creator%>\r\n" +
+      "Difficulty: <%wp.difficulty%>, Terrain: <%wp.terrain%>, FunFactor: <%wp.funfactor%>\r\n" +
+      "Lat: <%wp.latwritten%> Lon: <%wp.lonwritten%>\r\n" +
+      "Type: <%wp.type%> (<%wp.size%>) <%out.relativedistance%>\r\n" +
+      "Creation: <%out.cdate%>, Last comment: <%wp.mdays%> days ago (<%wp.comment0Type%>)\r\n" +
+      "\r\n<%wpText.shortdesc%>\r\n" +
+      "\r\n<%wpText.longdesc%>\r\n" +
+      "\r\n<%out.hint%>\r\n\r\n\r\n\r\n"
   },
 
 
@@ -215,7 +134,7 @@ N|O|P|Q|R|S|T|U|V|W|X|Y|Z</pre></font><br>",
     'desc'    =>     'Tab Delimited (GPS Connect)',
     'spacer'    => "",
     'templatePre' => "",
-    'templateWP'    => "<%out.id%>\t<%wp.latdata%>\t<%wp.londata%>\t0\n"
+    'templateWP'    => "<%out.id%>\t<%wp.latdata%>\t<%wp.londata%>\t0\r\n"
   },
 
   'csv'    => {
@@ -224,10 +143,10 @@ N|O|P|Q|R|S|T|U|V|W|X|Y|Z</pre></font><br>",
     'desc'    => 'CSV for spreadsheet imports',
     'spacer'    => "",
     'templatePre' => "\"Name\",\"Waypoint ID\",\"Creator\",\"Difficulty\",\"Terrain\"," +
-      "\"Latitude\",\"Longitude\",\"Type\",\"Creation Date\",\"Details\"\n",
+      "\"Latitude\",\"Longitude\",\"Type\",\"Creation Date\",\"Details\"\r\n",
     'templateWP'    => "\"<%wp.name%>\",\"<%out.wid%>\",\"<%wp.creator%>\"," +
       "<%wp.difficulty%>,<%wp.terrain%>,\"<%wp.latwritten%>\",\"<%wp.lonwritten%>\"," +
-      "\"<%wp.type%>\",\"<%wp.cdate%>\",\"<%out.details%>\"\n"
+      "\"<%wp.type%>\",\"<%out.cdate%>\",\"<%outText.details%>\"\r\n"
   },
 
   'vcf'    => {
@@ -368,7 +287,7 @@ N|O|P|Q|R|S|T|U|V|W|X|Y|Z</pre></font><br>",
     'desc'    => 'MapTech Exchange',
     'spacer'    => "\n",
     'templatePre' => '',
-    'templateWP' => "<%out.latdatapad5%>, <%out.londatapad5%>, \"<%wp.name%> by <%wp.creator%> (<%wp.type%> - <%wp.difficulty%>/<%wp.terrain%>)\", \"<%out.wid%>\", \"<%wp.name%> by <%wp.creator%> (<%wp.type%> - <%wp.difficulty%>/<%wp.terrain%>)\", ff0000, 47\n"
+    'templateWP' => "<%out.latdatapad5%>, <%out.londatapad5%>, \"<%wp.name%> by <%wp.creator%> (<%wp.type%> - <%wp.difficulty%>/<%wp.terrain%>)\", \"<%out.wid%>\", \"<%wp.name%> by <%wp.creator%> (<%wp.type%> - <%wp.difficulty%>/<%wp.terrain%>)\", ff0000, 47\r\n"
   },
 
 
@@ -387,12 +306,12 @@ N|O|P|Q|R|S|T|U|V|W|X|Y|Z</pre></font><br>",
     'ext'        => 'wpt',
     'mime'    => 'application/x-ozi-wpt',
     'desc'    => 'OziExplorer',
-    'spacer'    => "\n",
-    'templatePre' => "OziExplorer Waypoint File Version 1.1\n" +
-      "WGS 84\n" +
-      "Reserved 2\n" +
-      "Reserved 3\n",
-    'templateWP' => "<%out.counter%>,<%out.wid%>,<%out.latdatapad6%>,<%out.londatapad6%>,37761.29167,0,1,3,0,65535,<%wp.name%> by <%wp.creator%> (<%wp.type%> - <%wp.difficulty%>/<%wp.terrain%>),0,0,0,-777,6,0,17\n"
+    'spacer'    => "\r\n",
+    'templatePre' => "OziExplorer Waypoint File Version 1.1\r\n" +
+      "WGS 84\r\n" +
+      "Reserved 2\r\n" +
+      "Reserved 3\r\n",
+    'templateWP' => "<%out.counter%>,<%out.wid%>,<%out.latdatapad6%>,<%out.londatapad6%>,37761.29167,0,1,3,0,65535,<%wp.name%> by <%wp.creator%> (<%wp.type%> - <%wp.difficulty%>/<%wp.terrain%>),0,0,0,-777,6,0,17\r\n"
   },
 
   'tpg' => {
@@ -419,7 +338,7 @@ N|O|P|Q|R|S|T|U|V|W|X|Y|Z</pre></font><br>",
     'mime'    => 'application/gpsdrive',
     'spacer' => '',
     'desc'    => 'GpsDrive',
-    'spacer'    => "\n",
+    'spacer'    => "\r\n",
     'templatePre' => '',
     'templateWP' => "<%out.wid%> <%out.latdatapad5%> <%out.londatapad5%> Geocache\n"
   },
